-------------------------------------------------------------------------------
--=> Events.lua
-- weizPVP: Events
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
--> LuaCheck Settings
-- luacheck: no max line length
-- luacheck: ignore  _.* for k self
--!! DEBUG
-- luacheck: globals weizPVP_DEBUG_previousOutput ViragDevTool_AddData
--> weizPVP: Libs
-- luacheck: globals LibStub LibInspect ViragDevTool
--> weizPVP: Globals
-- luacheck: globals weizPVPEvents weizPVP weizPVP.Settings WolrdPVP_InspectCache weizPVPInspect
--> WoW API Globals
-- luacheck: globals InCombatLockdown GetUnitName UnitIsEnemy UnitIsPlayer UnitClass UnitRace UnitName
-- luacheck: globals UnitLevel GetGuildInfo UnitHealth UnitHealthMax GetInspectSpecialization GetSpecializationRoleByID GetSpecializationInfoByID
-- luacheck: globals truncate  CanInspect UnitGUID time floor GetInventoryItemLink GetInventorySlotInfo GetItemInfo GetTime
-- luacheck: globals UnitIsPVP UnitIsDeadOrGhost ClearInspectPlayer NotifyInspect GetRealmName UnitIsDead UnitIsGhost UnitReaction
-- luacheck: globals strsub strfind bit COMBATLOG_OBJECT_REACTION_HOSTILE C_NamePlate CombatLogGetCurrentEventInfo UnitExists
-- luacheck: globals COMBATLOG_OBJECT_TYPE_PLAYER CopyTable GetPlayerInfoByGUID GetZonePVPInfo IsInRaid IsInGroup
-- luacheck: globals UnitInRaid GetNumGroupMembers UnitInParty UnitPlayerOrPetInParty UnitDebuff SendChatMessage GetSpellLink
-- luacheck: globals GetNumRaidMembers GetSpellInfo IsInInstance RAID_CLASS_COLORS
-------------------------------------------------------------------------------
--=> GLOBALS TO LOCALS
-------------------------------------------------------------------------------
local _G = _G
local GetTime = _G.GetTime
local UnitIsEnemy = _G.UnitIsEnemy
local UnitIsPlayer = _G.UnitIsPlayer
local UnitClass = _G.UnitClass
local GetUnitName = _G.GetUnitName
local UnitName = _G.UnitName
local UnitLevel = _G.UnitLevel
local GetGuildInfo = _G.GetGuildInfo
local UnitIsPVP = _G.UnitIsPVP
local UnitIsDeadOrGhost = _G.UnitIsDeadOrGhost
local UnitHealth = _G.UnitHealth
local UnitHealthMax = _G.UnitHealthMax
local GetPlayerInfoByGUID = _G.GetPlayerInfoByGUID
local weizPVP = weizPVP

-------------------------------------------------------------------------------
--=> LOCALS
-------------------------------------------------------------------------------
local _

-------------------------------------------------------------------------------
--=> GLOBALS
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--=> GLOBAL FUNCTIONS
-------------------------------------------------------------------------------

--=> Update Player DB
--> All new entries and updates to weizPVP.PlayerDB happen here.
-----------------------------------------------------------------------
local lastAddedTime = 0
function weizPVP:UpdatePlayerDB(unitData)
    local nameAndRealm = unitData.nameAndRealm

    if nameAndRealm and unitData.name ~= "Unknown" then --> If we have a name and a timestamp; update data.
        if weizPVP.PlayerDB[nameAndRealm] then --> Update Existing entry if it exists
            --> UPDATE DBPreload

            weizPVP.PlayerDB[nameAndRealm].LastUpdated = GetTime()

            --> SET GUILD
            if weizPVP.PlayerDB[nameAndRealm].Guild and unitData.guild then
                if weizPVP.PlayerDB[nameAndRealm].Guild ~= unitData.guild then
                    weizPVP.PlayerDB[nameAndRealm].Guild = unitData.guild --> Update Guild if changed
                end
            end
            weizPVP.PlayerDB[nameAndRealm].Guild = unitData.guild or weizPVP.PlayerDB[nameAndRealm].Guild

            --> SET CLASS
            if not weizPVP.PlayerDB[nameAndRealm].Class and unitData.class then
                weizPVP.PlayerDB[nameAndRealm].Class = unitData.class
            end

            --> SET RACE
            if not weizPVP.PlayerDB[nameAndRealm].Race and unitData.race then
                weizPVP.PlayerDB[nameAndRealm].Race = unitData.race
            end

            -->  UPDAE ESTIMATED VALUES
            if weizPVP.PlayerDB[nameAndRealm].Estimated == true then --> Handle Estimations
                weizPVP.PlayerDB[nameAndRealm].Estimated = unitData.estimated

                if unitData.level and weizPVP.PlayerDB[nameAndRealm].Level then --> If we have an existing level and an updated value
                    if unitData.level > weizPVP.PlayerDB[nameAndRealm].Level then --> If new level is higher, use thats
                        weizPVP.PlayerDB[nameAndRealm].Level = unitData.level or weizPVP.PlayerDB[nameAndRealm].Level
                    end
                else
                    weizPVP.PlayerDB[nameAndRealm].Level = unitData.level or weizPVP.PlayerDB[nameAndRealm].Level
                end
            else --> UNIT DATA ALREAY ENTERED; CHECK FOR UPDATE IN LEVEL
                --> CHECK FOR LEVEL UPDATE
                if
                    not weizPVP.PlayerDB[nameAndRealm].Level and unitData.level and
                        weizPVP.PlayerDB[nameAndRealm].Level < unitData.level
                 then
                    weizPVP.PlayerDB[nameAndRealm].Level = unitData.level
                end
            end

            --> UPDATE CACHE
            local timestamp = GetTime()
            if lastAddedTime >= timestamp then
                timestamp = lastAddedTime + 0.001
            end
            lastAddedTime = timestamp

            if not weizPVP.PlayerActiveCache[nameAndRealm] then
                weizPVP.PlayerActiveCache[nameAndRealm] = {}

                weizPVP.PlayerActiveCache[nameAndRealm].TimeAdded = timestamp
            end
            weizPVP.PlayerActiveCache[nameAndRealm].LastUpdated = timestamp

            weizPVP.PlayerActiveCache[nameAndRealm].Estimated = weizPVP.PlayerDB[nameAndRealm].Estimated
            weizPVP.PlayerActiveCache[nameAndRealm].Class = weizPVP.PlayerDB[nameAndRealm].Class
            weizPVP.PlayerActiveCache[nameAndRealm].Race = weizPVP.PlayerDB[nameAndRealm].Race
            weizPVP.PlayerActiveCache[nameAndRealm].Guild = weizPVP.PlayerDB[nameAndRealm].Guild
            weizPVP.PlayerActiveCache[nameAndRealm].Level = weizPVP.PlayerDB[nameAndRealm].Level

            if unitData.healthPercent then --> HEALTH
                weizPVP.PlayerActiveCache[nameAndRealm].HealthPercent = unitData.healthPercent
            end

            if unitData.dead ~= nil then --> DEAD
                weizPVP.PlayerActiveCache[nameAndRealm].Dead = unitData.dead
            end

            if unitData.stealth ~= nil then --> STEALTH
                weizPVP.PlayerActiveCache[nameAndRealm].Stealth = unitData.stealth
            end

            --> UPDATE PLAYER LIST
            weizPVP:AddPlayerData(nameAndRealm, timestamp, unitData.dead)

            return true
        else --> Create new entry in DB (Player was not in DB)
            local timestamp = GetTime()
            if lastAddedTime >= timestamp then
                timestamp = lastAddedTime + 0.001
            end
            lastAddedTime = timestamp

            weizPVP.PlayerDB[nameAndRealm] = {
                ["Name"] = unitData.name,
                ["Estimated"] = unitData.estimated,
                ["Realm"] = unitData.realm,
                ["Guild"] = unitData.guild,
                ["Class"] = unitData.class,
                ["Dead"] = unitData.class,
                ["Race"] = unitData.race,
                ["Level"] = unitData.level,
                ["LastUpdated"] = timestamp
            }
            weizPVP.PlayerActiveCache[nameAndRealm] = {
                ["Name"] = unitData.name,
                ["Estimated"] = unitData.estimated,
                ["Realm"] = unitData.realm,
                ["Guild"] = unitData.guild,
                ["Class"] = unitData.class,
                ["Dead"] = unitData.dead,
                ["Race"] = unitData.race,
                ["Level"] = unitData.level,
                ["LastUpdated"] = timestamp,
                ["TimeAdded"] = timestamp,
                ["HealthPercent"] = unitData.healthPercent
            }
            weizPVP:AddPlayerData(nameAndRealm, timestamp, unitData.dead)
            return true
        end
    end
    return false
end

--=> Parse Unit Ability
-- Take data from Combat Log and process the data to determine more info on the unit involved
-----------------------------------------------------------------------
function weizPVP:ParseUnitAbility(event, player, spellName)
    local abilityType = strsub(event, 1, 5)
    if
        (player ~= nil and spellName ~= nil) and
            (abilityType == "SWING" or abilityType == "SPELL" or abilityType == "RANGE")
     then
        local name = player
        local class
        local race
        local specName
        local level = 0

        local playerDataOUT
        --> Check AbilityDB
        local ability = weizPVP.AbilityDB[spellName]

        --> Check SpecDB
        local specTable = weizPVP.SpecDB[spellName]
        if specTable then
            class = specTable[1]
            specName = specTable[2]
        end

        --> class
        if ability and ability.class then
            class = ability.class
        elseif specTable and specTable[1] then
            class = specTable[1]
        end
        --> level
        if ability and ability.level then
            level = ability.level
        end
        --> race
        if ability and ability.race then
            race = ability.race
        end

        --> specName
        if specTable and specTable[2] then
            specName = specTable[2]
        end

        playerDataOUT = {
            ["name"] = name,
            ["class"] = class,
            ["level"] = level,
            ["race"] = race,
            ["specName"] = specName,
            ["ability"] = spellName
        }

        --> RETURN DATA
        if playerDataOUT.class or playerDataOUT.level or playerDataOUT.race or playerDataOUT.specName then
            return true, playerDataOUT
        else
            return false, {nil, nil, nil, nil, nil, nil}
        end
    end
    --> no player name or spell given; we can't do shit
    return false, {nil, nil, nil, nil, nil, nil}
end

--=> GET UNIT DATA
-----------------------------------------------------------------------
--> Gets data for events with a UNITID, such as 'target' or 'nameplate'
function weizPVP:GetUnitData(unit)
    --> Confirm Unit is a player that is an enemy or attackable
    local unitEnemy = false
    if UnitIsEnemy("player", unit) and UnitIsPlayer(unit) then
        unitEnemy = true
    end
    if unitEnemy then
        if not InCombatLockdown() then --> Don't parse all data in large fights to improve performance
            --> GET NAME & REALM
            local nameAndRealm = GetUnitName(unit, true)
            local estimated = false
            local name, realm = UnitName(unit, true)
            if not realm then
                realm = weizPVP.CurrentRealm -- set to myrealm name if we got nil back from UnitName
            end

            --> Timestamp
            local timestamp = GetTime()

            --> GET CLASS
            local class = UnitClass(unit)
            class = string.upper(class)
            class = string.gsub(class, "%s", "")

            --> GET RACE
            local race = select(1, UnitRace(unit))

            --> GET LEVEL
            local level = tonumber(UnitLevel(unit))

            --> SET STEALTH
            local stealth = false

            --> GET GUILD
            local guild = GetGuildInfo(unit)

            --> GET PVP FLAGGED
            local pvp = UnitIsPVP(unit)

            --> GET DEAD OR GHOST
            local dead = UnitIsDeadOrGhost(unit)

            --> GET HEALTH
            local healthPercent = UnitHealth(unit) / UnitHealthMax(unit) * 100

            local dataOUT = {
                ["nameAndRealm"] = nameAndRealm,
                ["name"] = name,
                ["estimated"] = estimated,
                ["realm"] = realm,
                ["guild"] = guild,
                ["class"] = class,
                ["race"] = race,
                ["level"] = level,
                ["timestamp"] = timestamp,
                ["pvp"] = pvp,
                ["dead"] = dead,
                ["stealth"] = stealth,
                ["healthPercent"] = healthPercent
            }

            return dataOUT
        else --> Get All Data; low numbers of players detected
            --> GET NAME & REALM
            local nameAndRealm = GetUnitName(unit, true)
            local estimated = false
            local name, realm = UnitName(unit, true)
            if not realm then
                realm = weizPVP.CurrentRealm --> set to myrealm name if we got nil back from UnitName
            end

            --> Timestamp
            local timestamp = GetTime()

            --> GET HEALTH
            local healthPercent = UnitHealth(unit) / UnitHealthMax(unit) * 100

            local dataOUT = {
                ["nameAndRealm"] = nameAndRealm,
                ["name"] = name,
                ["estimated"] = estimated,
                ["realm"] = realm,
                ["guild"] = GetGuildInfo(unit),
                ["level"] = tonumber(UnitLevel(unit)),
                ["timestamp"] = timestamp,
                ["pvp"] = UnitIsPVP(unit),
                ["dead"] = UnitIsDeadOrGhost(unit),
                ["healthPercent"] = healthPercent
            }

            return dataOUT
        end
    end

    return nil
end

-------------------------------------------------------------------------------
--=> EVENT FUNCTIONS
-------------------------------------------------------------------------------

--=> EVENT: Player Mouseover
-----------------------------------------------------------------------
function weizPVP:PlayerMouseoverEvent()
    --> MOUSEOVER UNIT
    if UnitIsPlayer("mouseover") and UnitIsEnemy("player", "mouseover") then
        local unitData = weizPVP:GetUnitData("mouseover")
        if unitData then
            weizPVP:UpdatePlayerDB(unitData)
        end
    end
end

--=> EVENT: Player Focus Changed
-----------------------------------------------------------------------
function weizPVP:PlayerFocusEvent()
    --> FOCUS INFO
    if UnitIsPlayer("focus") and UnitIsEnemy("player", "focus") then
        local unitData = weizPVP:GetUnitData("focus")
        if unitData then
            weizPVP:UpdatePlayerDB(unitData)
        end
    end
end

--=> EVENT: Player Target Changed
-----------------------------------------------------------------------
function weizPVP:PlayerTargetEvent()
    --> TARGET INFO
    if UnitIsPlayer("target") and UnitIsEnemy("player", "target") then
        local unitData = weizPVP:GetUnitData("target")
        if unitData then
            weizPVP:UpdatePlayerDB(unitData)
        end
    end
end

--=> EVENT: UNIT Target Changed: UNIT_TARGET
--> Replaes targets of unit events for mouseovers/targets/focuses etc
-----------------------------------------------------------------------
function weizPVP:UnitTargetEvent(_, unit)
    --> UNIT INFO
    if UnitIsPlayer(unit) and UnitIsEnemy("player", unit) then
        local unitData = weizPVP:GetUnitData(unit)
        if unitData then
            weizPVP:UpdatePlayerDB(unit)
        end
    end
    --> TARGET OF UNIT INFO
    if UnitIsPlayer(unit .. "target") then
        local unitData = weizPVP:GetUnitData(unit .. "target")
        if unitData then
            weizPVP:UpdatePlayerDB(unitData)
        end
    end
    --> TARGET OF TARGET OF UNIT INFO
    if UnitIsPlayer(unit .. "target" .. "target") then
        local unitData = weizPVP:GetUnitData(unit .. "target" .. "target")
        if unitData then
            weizPVP:UpdatePlayerDB(unitData)
        end
    end
end

--=> EVENT: UNIT_HEALTH
-----------------------------------------------------------------------
function weizPVP:UnitHealth(_, unit, _)
    if UnitIsEnemy("player", unit) and UnitIsPlayer(unit) then
        local healthCurrent = UnitHealth(unit)
        local healthMax = UnitHealthMax(unit)
        local healthPercent = healthCurrent / healthMax * 100

        local nameAndRealm = GetUnitName(unit, true)
        local dead = UnitIsDeadOrGhost(unit)
        if healthPercent <= 0 then
            dead = true
        end

        if weizPVP.PlayerActiveCache[nameAndRealm] and nameAndRealm and (healthPercent or dead) then
            --> Should be updating PlayerCache, not the DB for health updates
            weizPVP.PlayerActiveCache[nameAndRealm].HealthPercent = healthPercent
            weizPVP.PlayerActiveCache[nameAndRealm].Dead = dead
            weizPVP.PlayerActiveCache[nameAndRealm].LastUpdated = GetTime()

            --> UPDATE PLAYER LIST
            weizPVP:AddPlayerData(nameAndRealm, GetTime(), dead)
        else
            local unitData = weizPVP:GetUnitData(unit)
            if unitData then
                weizPVP:UpdatePlayerDB(unitData)
            end
        end
    end
end

--=> EVENT: CCOMBAT_LOG_UNFILTERED
-----------------------------------------------------------------------
function weizPVP:CombatLogEvent()
    local timestamp,
        event,
        _,
        srcGUID,
        srcName,
        srcFlags,
        _,
        dstGUID,
        dstName,
        dstFlags,
        _,
        _,
        arg2,
        _,
        _,
        _,
        _,
        _,
        _,
        _,
        _ = CombatLogGetCurrentEventInfo()

    --> Dont scan yourself
    if dstName == srcName and srcName == weizPVP.CurrentCharName then
        return
    end

    --> Source INFO
    local isPlayer = bit.band(srcFlags, COMBATLOG_OBJECT_TYPE_PLAYER) == COMBATLOG_OBJECT_TYPE_PLAYER
    local isHostile = bit.band(srcFlags, COMBATLOG_OBJECT_REACTION_HOSTILE) == COMBATLOG_OBJECT_REACTION_HOSTILE
    if isPlayer and isHostile and srcGUID and srcName then
        local srcType = strsub(srcGUID, 1, 6)

        --> The unit is a player
        if srcType == "Player" then
            --> Parse Ability and find class/race/level
            --> weizPVP:ParseUnitAbility(event, player, flags, spellId, spellName)
            if weizPVP.PlayerDB[srcName] and weizPVP.PlayerDB[srcName].Estimated == true then
                local parsedAbility, parsedAbilityData = weizPVP:ParseUnitAbility(event, srcName, arg2)
                if parsedAbility and parsedAbilityData then --> If we got data from parsing ability and player isnt in cache, add player to cache
                    local name = string.gsub(srcName, " %- ", "-")
                    local class = select(2, GetPlayerInfoByGUID(srcGUID))
                    local stealth
                    local zoneInfo = select(1, GetZonePVPInfo())
                    class = class or parsedAbilityData.class
                    if
                        event == "SPELL_AURA_APPLIED" and ((arg2 == "Stealth") or (arg2 == "Prowl")) and srcName and
                            (tostring(zoneInfo) ~= "sanctuary true")
                     then
                        stealth = true
                        weizPVP:ShowAlert("Stealth Detected!", srcName, {["r"] = 0, ["b"] = 0, ["g"] = 0, ["a"] = 0.8})
                    end
                    local unitData = {
                        ["nameAndRealm"] = srcName,
                        ["name"] = name,
                        ["estimated"] = true,
                        ["class"] = class,
                        ["stealth"] = stealth,
                        ["race"] = parsedAbilityData.race,
                        ["level"] = parsedAbilityData.level,
                        ["timestamp"] = timestamp
                    }

                    weizPVP:UpdatePlayerDB(unitData)
                end
            else --> EXISTING UNIT; UPDATE
                local name = string.gsub(srcName, " %- ", "-")
                local stealth
                local zoneInfo = select(1, GetZonePVPInfo())
                if
                    event == "SPELL_AURA_APPLIED" and (arg2 == "Stealth") or
                        (arg2 == "Prowl") and srcName and (tostring(zoneInfo) ~= "sanctuary true")
                 then
                    stealth = true
                    weizPVP:ShowAlert("Stealth Detected!", srcName, {["r"] = 0, ["b"] = 0, ["g"] = 0, ["a"] = 0.8})
                end
                local unitData = {
                    ["nameAndRealm"] = srcName,
                    ["name"] = name,
                    ["stealth"] = stealth,
                    ["estimated"] = true,
                    ["timestamp"] = timestamp
                }

                weizPVP:UpdatePlayerDB(unitData)
            end
        end
    end

    --> Destination
    isPlayer = bit.band(dstFlags, COMBATLOG_OBJECT_TYPE_PLAYER) == COMBATLOG_OBJECT_TYPE_PLAYER
    isHostile = bit.band(dstFlags, COMBATLOG_OBJECT_REACTION_HOSTILE) == COMBATLOG_OBJECT_REACTION_HOSTILE
    if isPlayer and isHostile and dstGUID and dstName then
        local dstType = strsub(dstGUID, 1, 6)

        if dstType == "Player" then
            --> Parse Ability and find class/race/level
            --> weizPVP:ParseUnitAbility(analyseSpell, event, player, flags, spellId, spellName)
            if weizPVP.PlayerDB[dstName] and weizPVP.PlayerDB[dstName].Estimated == true then --> ESITMATED UNIT; GET INFO AND ADD
                local parsedAbility, parsedAbilityData = weizPVP:ParseUnitAbility(event, srcName, arg2)
                if parsedAbility then --> If we got data from parsing ability, update PlayerCache
                    local name = string.gsub(dstName, " %- ", "-")
                    local class = select(2, GetPlayerInfoByGUID(dstGUID))
                    class = class or parsedAbilityData.class
                    local dead
                    if event == "UNIT_DIED" then
                        dead = true
                    end
                    local unitData = {
                        ["nameAndRealm"] = dstName,
                        ["name"] = name,
                        ["estimated"] = true,
                        ["class"] = class,
                        ["dead"] = dead,
                        ["stealth"] = false,
                        ["race"] = parsedAbilityData.race,
                        ["level"] = parsedAbilityData.level,
                        ["timestamp"] = timestamp
                    }

                    weizPVP:UpdatePlayerDB(unitData)
                end
            else --> KNOWN UNIT: UPDATE
                -- local name = string.gsub(dstName, " %- ", "-")
                local dead
                if event == "UNIT_DIED" then
                    dead = true
                end
                local unitData = {
                    ["nameAndRealm"] = dstName,
                    -- ["name"] = name,
                    ["dead"] = dead,
                    ["estimated"] = true,
                    ["timestamp"] = timestamp
                }

                weizPVP:UpdatePlayerDB(unitData)
            end
        end
    end
end

--=> EVENT: NAMEPLATE_ADDED
-- ====================================================
function weizPVP:NameplateAdded(_, unit)
    local unitData = weizPVP:GetUnitData(unit)
    if unitData then
        weizPVP:UpdatePlayerDB(unitData)
    end
end

--=> EVENT: NAMEPLATE REMOVED
-- ====================================================
function weizPVP:NameplateRemoved(_, unit)
    local unitData = weizPVP:GetUnitData(unit)
    if unitData then
        weizPVP:UpdatePlayerDB(unitData)
    end
end

--=> EVENT: ENTERING COMBAT
-----------------------------------------------------------------------
function weizPVP:EnteringCombat()
    weizPVP:SetStatusBarMessage("|cFFFF0000 + IN COMBAT|r", false, 0)
end

--=> EVENT: LEAVING COMBAT
-----------------------------------------------------------------------
function weizPVP:LeavingCombat()
    weizPVP:SetStatusBarMessage("|cFF42f489 - OUT OF COMBAT|r", true, 3)
end

--=> EVENT: Left Combat
-----------------------------------------------------------------------
function weizPVP:LeftCombatEvent()
end

--=> EVENT: Player Died
-----------------------------------------------------------------------
function weizPVP:PlayerDeadEvent()
end

--=> EVENT: Channel Notice
-----------------------------------------------------------------------
function weizPVP:ChannelNoticeEvent()
end

--=> EVENT: Zone Changed
-----------------------------------------------------------------------
function weizPVP:ZoneChangedEvent()
    -- weizPVP:ClearListData()
    -- if ViragDevTool then
    --     ViragDevTool_AddData("ZoneChangedEvent", "ZoneChangedEvent")
    -- end
end

--=> EVENT: Zone Changed + New Area
-----------------------------------------------------------------------
function weizPVP:ZoneChangedNewAreaEvent()
    -- weizPVP:ClearListData()
    -- if ViragDevTool then
    --     ViragDevTool_AddData("ZoneChangedNewAreaEvent", "ZoneChangedNewAreaEvent")
    -- end
end

--=> EVENT: Player Entering World
-----------------------------------------------------------------------
function weizPVP:PlayerEnteringWorldEvent()
end

--=> EVENT: Player Levled Up
--> Update player level
-----------------------------------------------------------------------
function weizPVP:PlayerLevelUp()
    weizPVP.PlayerLevel = UnitLevel("player")
end

--=> EVENT: Spell Cast Succeeded
-----------------------------------------------------------------------

-->Check for Lust
-----------------------------------------------------------------------
local lustSpells = {
    [146555] = true, --> Drums of Rage 25% Haste Buff => Exhaustion Debuff (Castable by Anyone)
    [178207] = true, --> Drums of Fury 25% Haste Buff => Exhaustion Debuff (Castable by Anyone)
    [230935] = true, --> Drums of the Mountain 25% Haste Buff => Exhaustion Debuff (Castable by Anyone)
    [2825] = true, --> Bloodlust 30% Haste Buff => Sated Debuff (Castable by Horde Shaman)
    [32182] = true, --> Heroism 30% Haste Buff => Exhaustion Debuff (Castable by Alliance Shaman)
    [80353] = true, --> Time Warp 30% Haste Buff => Temporal Displacement Debuff (Castable by Mage)
    [90355] = true, --> Ancient Hysteria 30% Haste Buff => Insanity Debuff (Castable by Hunter Core Hound Pet)
    [160452] = true --> Netherwinds 30% Haste Buff => Fatigued Debuff (Castable by Hunter Nether Ray Pet)
}
function weizPVP:SpellCastSucceeded(_, unit, _, spellID, _, _, _, _)
    -- --> CREATE UNIT NAME WITH CLASS COLOR
    -- local name = UnitName(unit)
    -- local class = UnitClass(unit)
    -- class = string.upper(class)
    -- class = string.gsub(class, "%s", "")

    -- --=> LUST CHECK
    -- --> Check to see if someone lusted and print to chat
    -- ----------------------------------
    -- --> First see if we're even in a group
    -- local numGroupMembers = GetNumGroupMembers()
    -- local solo
    -- if numGroupMembers > 0 then
    --     solo = false
    -- else
    --     solo = true
    -- end
    -- --> But was the luster in our group?

    -- GetNumPartyMembers
    -- local unitNameColored = name
    -- if RAID_CLASS_COLORS[class] ~= nil and name then
    --     unitNameColored = RAID_CLASS_COLORS[class]:WrapTextInColorCode(name)
    -- end

    -- local spellLink = GetSpellLink(spellID)

    -- if spellLink == nil then
    --     spellLink = "|cff00ffe5[UNKNOWN]|r"
    -- end

    -- -- local IsInRaid = IsInRaid --Raid and not Party
    -- -- local IsInGroup = IsInGroup -- Raid or Party
    -- -- local IsInInstance = IsInInstance -- Instane Group


    -- if lustSpells[spellID] and name then --> Continue only if we see lust
    -- -- if name then
    --     -- if IsInGroup(LE_PARTY_CATEGORY_INSTANCE) then --> INSTANCE
    --     if IsInInstance() and not solo then --> INSTANCE
    --         SendChatMessage(unitNameColored .. "LUSTED with " .. spellLink, "INSTANCE_CHAT")
    --     elseif IsInGroup() and not IsInRaid() and not solo then --> PARTY
    --         SendChatMessage(unitNameColored .. "LUSTED with " .. spellLink, "RAID")
    --     elseif IsInRaid() and not solo then --> RAID
    --         SendChatMessage(unitNameColored .. "LUSTED with " .. spellLink, "PARTY")
    --     else --> SOLO
    --         SendChatMessage(unitNameColored .. "LUSTED with " .. spellLink, "SAY")
    --     end
    -- end
end

--=> EVENT: Aura Updated
--TODO: ADD DETECTION OF BOUNTY DEBUFF CREATE ALERT
-----------------------------------------------------------------------
function weizPVP:AuraUpdate()
end

--=> EVENT: Stealth Updated

-----------------------------------------------------------------------
function weizPVP:UpdatedStealth()
    local timestamp,
        event,
        arg1,
        srcGUID,
        srcName,
        srcFlags,
        arg2,
        dstGUID,
        dstName,
        dstFlags,
        _,
        _,
        _,
        _,
        _,
        _,
        _,
        _,
        _,
        _,
        _ = CombatLogGetCurrentEventInfo()

    --!! ---------------------------------------
    if ViragDevTool then
        ViragDevTool_AddData(
            {
                ["arg1"] = arg1,
                ["srcName"] = srcName,
                ["srcFlags"] = srcFlags,
                ["arg2"] = arg2,
                ["srcGUID"] = srcGUID
            },
            "UpdatedStealth"
        )
    end
    --!! ---------------------------------------
end

--=> EVENT: MinimapPing

-----------------------------------------------------------------------
function weizPVP:MinimapPing(event, unit, x, y)
    --!! ---------------------------------------
    if ViragDevTool then
        ViragDevTool_AddData(
            {
                ["unit"] = unit,
                ["x"] = x,
                ["y"] = y
            },
            "MinimapPing"
        )
    end
    --!! ---------------------------------------
end
