-------------------------------------------------------------------------------
--=> weizPVP.lua
-- weizPVP CORE
-------------------------------------------------------------------------------
--> LuaCheck Settings
-- luacheck: no max line length
-- luacheck: ignore  _.* for k self
--> DEBUG GLOBALS
-- luacheck: globals ViragDevTool_AddData
--> weizPVP Globals
-- luacheck: globals weizPVP
--> Libs
-- luacheck: globals LibStub
--> WoW Globals
-- luacheck: globals table.wipe GetRealmName GetUnitName UnitLevel UnitFactionGroup GetCVar GetNetStats
-------------------------------------------------------------------------------
--=> LIBS
-------------------------------------------------------------------------------
weizPVP = LibStub("AceAddon-3.0"):NewAddon("weizPVP", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
-- weizPVP:RegisterChatCommand("mychat", "SlashCommands")
local SM = LibStub:GetLibrary("LibSharedMedia-3.0")

--> Minimap/LDB
-----------------------------------------------------------
-- local miniMapIcon = LibStub("LibDBIcon-1.0")
-- local weizPVP_LDB =
--     LibStub("LibDataBroker-1.1"):NewDataObject(
--     "weizPVP",
--     {
--         type = "data source",
--         text = "weizPVP",
--         icon = "Interface\\Addons\\weizPVP\\Icons\\wpvp_icon.tga",
--         OnClick = function()
--             weizPVP:MiniMapToggle()
--         end
--     }
-- )
-- local LDBIcon = LibStub("LibDBIcon-1.0")

-------------------------------------------------------------------------------
--=> GLOBALS TO LOCALS
-------------------------------------------------------------------------------
local table_wipe = table.wipe

-------------------------------------------------------------------------------
--=> GLOBALS
-------------------------------------------------------------------------------
--> Settings
weizPVP.Settings = {} --> Create blank global table for weizPVP Settings
--> Player Tables
weizPVP.PlayerDB = {} --> player db of detected players
weizPVP.PlayerActiveCache = {} --> Cache of players while active

--> Player Counts
weizPVP.NearbyCount = 0
weizPVP.ListCount = 0
weizPVP.ListVisableCount = 0

--> WoW Settings
weizPVP.ParticleDensity = GetCVar("particleDensity") -- 10 to 100
weizPVP.Latency = select(4, GetNetStats()) -- World Latency

-------------------------------------------------------------------------------
--=> Default Settings
-------------------------------------------------------------------------------
    weizPVP.Settings = {
        Enabled = true,
        Shown = true,
        --> Resource Usage
        ResourceUsage = {
            High = false,
            MaxShownPlayers = 28,
        },
        --> BG/BORDER FOR ALL FRAMES
        BACKGROUNDTEXTURE = "weizPVP-Background",
        BORDERTEXTURE = "weizPVP-Border",
        BACKGROUNDCOLOR = {
            R = 0,
            G = 0,
            B = 0,
            A = 0.7
        },
        --> MAIN FRAME
        MAIN = {
            WIDTH = 260,
            HEIGHT = 0,
            BORDER_SIZE = 2
        },
        --> SCROLL FRAME
        SCROLL = {
            HEIGHT = 0,
            WIDTH = 0,
            scrollBarWidth = 4,
            SCROLLOFFET = 0
        },
        --> LIST FRAME
        LIST = {
            ROWHEIGHT = 0,
            HEIGHT = 0,
            WIDTH = 0
        },
        --> Header Settings
        HEADER = {
            HEIGHT = 34,
            WIDTH = 0,
            TEXT = {
                FONT = "Roboto Condensed BoldItalic",
                FONTSIZE = 20,
                FONTOUTLINE = "OUTLINE"
            },
            BUTTONS = {
                CONFIG = true,
                STATS = true,
                CLEAR = true,
                PREV = true,
                NEXT = true,
                CLOSE = true
            }
        },
        --> FOOTER FRAME
        FOOTER = {
            HEIGHT = 22,
            TEXT = {
                -- FONT = "Accidental Presidency",
                FONT = "Roboto Condensed Bold",
                FONTSIZE = 16,
                FONTOUTLINE = "OUTLINE"
            }
        },
        --> BARS
        BARS = {
            ALPHA_DEFAULT = 1,
            ALPHA_INACTIVE = 0.5,
            ALPHA_DEAD = 0.5,
            MAXNUMBER = 24,
            HEIGHT = 18,
            WIDTH = 0,
            SPACING = 1,
            NUM_DISPLAYED = 12,
            TEXTURE = "weizPVP.Bar",
            HIGHLIGHTTEXTURE = "weizPVP.Bar-Highlight",
            BGTEXTURE = "weizPVP.BarBackground",
            NAME_FONT_DIFF = -2,
            NAME_ALPHA_STEALTH = 0.6,
            LEVEL_FONT_DIFF = -2,
            GUILD_FONT_DIFF = -4
        },
        --> FONTS
        FONTS = {
            NARROW = "Accidental Presidency",
            NORMAL = "Roboto Condensed Bold",
            BOLD = "Roboto Condensed Bold",
            ITALIC = "Accidental Presidency Italic",
            BOLDITALIC = "Roboto Condensed BoldItalic"
        }
    }

    --=> Computed Settings
    -------------------------------------------------------------------------------
    weizPVP.Settings.LIST.ROWHEIGHT = weizPVP.Settings.BARS.HEIGHT + weizPVP.Settings.BARS.SPACING
    weizPVP.Settings.SCROLL.HEIGHT = (weizPVP.Settings.LIST.ROWHEIGHT * weizPVP.Settings.BARS.NUM_DISPLAYED) + weizPVP.Settings.BARS.SPACING
    weizPVP.Settings.SCROLL.WIDTH = weizPVP.Settings.MAIN.WIDTH
    weizPVP.Settings.MAIN.HEIGHT = weizPVP.Settings.SCROLL.HEIGHT
    weizPVP.Settings.LIST.HEIGHT = weizPVP.Settings.LIST.ROWHEIGHT * weizPVP.Settings.BARS.MAXNUMBER
    weizPVP.Settings.LIST.WIDTH = weizPVP.Settings.SCROLL.WIDTH
    weizPVP.Settings.BARS.WIDTH = weizPVP.Settings.LIST.WIDTH
    weizPVP.Settings.HEADER.WIDTH = weizPVP.Settings.MAIN.WIDTH



-------------------------------------------------------------------------------
--=> OPTIONS
-------------------------------------------------------------------------------
weizPVP.options = {
    name = "weizPVP Options",
    type = "group",
    args = {
        General = {
            name = "General Settings",
            desc = "General Settings",
            type = "group",
            order = 1,
            args = {
                intro1 = {
                    name = "weizPVP Settings",
                    type = "description",
                    order = 1
                },
                Enabled = {
                    name = "Enable",
                    desc = "Enable weizPVP",
                    type = "toggle",
                    order = 2,
                    width = "full",
                    get = function()
                        weizPVP.db.profile.Enabled = true
                        weizPVP.Enabled = true

                        return
                    end,
                    set = function(_, value)
                        if value == true then
                            weizPVP:OnEnable()
                        else
                            weizPVP:OnDisable()
                        end
                        -- weizPVP:EnableSpy(value, true)
                    end
                }
            }
        },
        DisplayOptions = {
            name = "Display Options",
            desc = "Display Options2",
            type = "group",
            order = 2,
            args = {
                intro = {
                    name = "Change options related to the addon's display",
                    type = "description",
                    order = 1
                },
                ShowOnDetection = {
                    name = "Show on Detection",
                    desc = "Show on Detection2",
                    type = "toggle",
                    order = 2,
                    width = "full",
                    get = function(info)
                        -- return weizPVP.db.profile.ShowOnDetection
                    end,
                    set = function(info, value)
                        -- weizPVP.db.profile.ShowOnDetection = value
                    end
                },
                Lock = {
                    name = "Lock the weizPVP window",
                    desc = "Lock the weizPVP window",
                    type = "toggle",
                    order = 5,
                    width = "full",
                    get = function(info)
                        -- return weizPVP.db.profile.Locked
                    end,
                    set = function(info, value)
                        -- weizPVP.db.profile.Locked = value
                        -- weizPVP:LockWindows(value)
                    end
                }
                -- SelectFont = {
                --     type = "select",
                --     order = 10,
                --     name = L["SelectFont"],
                --     desc = L["SelectFontDescription"],
                --     values = {
                --         ["2002"] = L["2002"],
                --         ["2002 Bold"] = L["2002 BOLD"],
                --         ["Arial Narrow"] = L["ARIAL NARROW"],
                --         ["Friz Quadrata TT"] = L["FRIZ QUADRATA TT"],
                --         ["FrizQuadrataCTT"] = L["FRIZQUADRATACTT"],
                --         ["MoK"] = L["MOK"],
                --         ["Morpheus"] = L["MORPHEUS"],
                --         ["Nimrod MT"] = L["NIMROD MT"],
                --         ["Skurri"] = L["SKURRI"]
                --     },
                --     get = function()
                --         return weizPVP.db.profile.Font
                --     end,
                --     set = function(info, value)
                --         weizPVP.db.profile.Font = value
                --         if value then
                --             weizPVP:UpdateBarTextures()
                --         end
                --     end
                -- },
                -- RowHeight = {
                --     type = "range",
                --     order = 11,
                --     name = L["RowHeight"],
                --     desc = L["RowHeightDescription"],
                --     min = 8,
                --     max = 20,
                --     step = 1,
                --     get = function()
                --         return weizPVP.db.profile.MainWindow.RowHeight
                --     end,
                --     set = function(info, value)
                --         weizPVP.db.profile.MainWindow.RowHeight = value
                --         if value then
                --             weizPVP:BarsChanged()
                --         end
                --     end
                -- }
            }
        },
        AlertOptions = {
            name = "Alert Options",
            desc = "Alert Options",
            type = "group",
            order = 3,
            args = {
                intro = {
                    name = "Options for alerts and notifications",
                    type = "description",
                    order = 1
                },
                WarnOnStealth = {
                    name = "WarnOnStealth",
                    desc = "WarnOnStealthDescription",
                    type = "toggle",
                    order = 7,
                    width = "full",
                    get = function(info)
                        -- return weizPVP.db.profile.WarnOnStealth
                    end,
                    set = function(info, value)
                        -- weizPVP.db.profile.WarnOnStealth = value
                    end
                },
                WarnOnStealthSound = {
                    name = "WarnOnStealthSound",
                    desc = "WarnOnStealthSoundDescription",
                    type = "toggle",
                    order = 5,
                    width = "full",
                    get = function(info)
                        -- return weizPVP.db.profile.EnableSound
                    end,
                    set = function(info, value)
                        -- weizPVP.db.profile.EnableSound = value
                    end
                }
            }
        },
        MinimapOptions = {
            name = "MinimapOptions",
            desc = "MinimapOptions",
            type = "group",
            order = 5,
            args = {
                intro = {
                    name = "MinimapOptionsDescription",
                    type = "description",
                    order = 1
                },
                MinimapButtonShow = {
                    name = "MinimapButtonShow",
                    desc = "MinimapButtonShow",
                    type = "toggle",
                    order = 2,
                    width = "full",
                    get = function(info)
                        return weizPVP.db.profile.minimap.hide
                    end,
                    set = function(info, value)
                        if value == true then
                            miniMapIcon:Show()
                        else
                            miniMapIcon:Hide()
                        end
                    end
                }
            }
        },
        DataOptions = {
            name = "DataOptions",
            desc = "DataOptions",
            type = "group",
            order = 6,
            args = {
                intro = {
                    name = "DataOptionsDescription",
                    type = "description",
                    order = 1
                },
                ResetPlayerDB = {
                    name = "ResetPlayerDB",
                    desc = "ResetPlayerDB",
                    type = "toggle",
                    order = 3,
                    width = "full",
                    get = function(info)
                        -- return weizPVP.db.profile.PurgeKoS
                    end,
                    set = function(info, value)
                        -- weizPVP.db.profile.PurgeKoS = value
                    end
                },
                ResetAddonSettings = {
                    name = "ResetAddonSettings",
                    desc = "ResetAddonSettings",
                    type = "toggle",
                    order = 6,
                    width = "full",
                    get = function(info)
                        -- return weizPVP.db.profile.UseData
                    end,
                    set = function(info, value)
                        -- weizPVP.db.profile.UseData = value
                    end
                }
            }
        }
    }
}

-------------------------------------------------------------------------------
--=> Slash Settings
-------------------------------------------------------------------------------
-- weizPVP.optionsSlash = {
--     name = "SlashCommand",
--     order = -3,
--     type = "group",
--     args = {
--         intro = {
--             name = "weizPVP Slash Commands",
--             type = "description",
--             order = 1,
--             cmdHidden = true
--         },
--         enable = {
--             name = "Enable weizPVP",
--             desc = "Enable weizPVP",
--             type = "execute",
--             order = 2,
--             func = function()
--                 -- Spy:EnableSpy(true, true)
--             end,
--             dialogHidden = true
--         },
--         config = {
--             name = "Config",
--             desc = "Open Configuration Options",
--             type = "execute",
--             order = 4,
--             func = function()
--                 -- weizPVP:ShowConfig()
--             end,
--             dialogHidden = true
--         }
--     }
-- }

-------------------------------------------------------------------------------
--=> SHARED MEDIA ASSETS
-------------------------------------------------------------------------------
--> Status Bars
SM:Register("statusbar", "weizPVP.Bar", [[Interface\Addons\weizPVP\Textures\bar.tga]])
SM:Register("statusbar", "weizPVP.Bar-Highlight", [[Interface\Addons\weizPVP\Textures\bar-highlight.tga]])
--> Bakgrounds
SM:Register("background", "weizPVP-Background", [[Interface\Addons\weizPVP\Textures\bg.tga]])
SM:Register("background", "weizPVP.BarBackground", [[Interface\Addons\weizPVP\Textures\bar_bg.tga]])
--> Borders
SM:Register("border", "weizPVP-Border", [[Interface\Addons\weizPVP\Textures\border.tga]])
--> Fonts
SM:Register("font", "Roboto Condensed Bold", [[Interface\Addons\weizPVP\Fonts\RobotoCondensed-Bold.ttf]])
SM:Register("font", "Roboto Condensed BoldItalic", [[Interface\Addons\weizPVP\Fonts\RobotoCondensed-BoldItalic.ttf]])
SM:Register("font", "Pepsi", [[Interface\Addons\weizPVP\Fonts\Pepsi.ttf]])
SM:Register("font", "Accidental Presidency", [[Interface\Addons\weizPVP\Fonts\Accidental-Presidency.ttf]])
SM:Register("font", "Accidental Presidency Italic", [[Interface\Addons\weizPVP\Fonts\Accidental-Presidency-Italic.ttf]])
SM:Register("font", "Open Sans Condensed", [[Interface\Addons\weizPVP\Fonts\OpenSansCondensed.ttf]])
SM:Register("font", "Open Sans Condensed Bold", [[Interface\Addons\weizPVP\Fonts\OpenSans-CondBold.ttf]])
SM:Register("font", "Ubuntu Condensed", [[Interface\Addons\weizPVP\Fonts\Ubuntu-C.ttf]])
SM:Register("font", "YanoneKaffeesatz Regular", [[Interface\Addons\weizPVP\Fonts\YanoneKaffeesatz-Regular.ttf]])
SM:Register("font", "Big Noodle Titling", [[Interface\Addons\weizPVP\Fonts\big_noodle_titling.ttf]])
SM:Register("font", "Source Sans Pro SemiBold", [[Interface\Addons\weizPVP\Fonts\SourceSansPro-SemiBold.ttf]])
SM:Register("font", "Source Sans Pro Bold", [[Interface\Addons\weizPVP\Fonts\SourceSansPro-Bold.ttf]])
SM:Register("font", "Source Sans Pro BoldItalic", [[Interface\Addons\weizPVP\Fonts\SourceSansPro-BoldItalic.ttf]])

--> Sounds
--> Sound Alarms
SM:Register("sound", "alarm-1", [[Interface\Addons\weizPVP\Sounds\alarm-1.ogg]])
SM:Register("sound", "alarm-2", [[Interface\Addons\weizPVP\Sounds\alarm-2.ogg]])
--> Sound Alerts
SM:Register("sound", "alert-1", [[Interface\Addons\weizPVP\Sounds\alert-1.ogg]])
SM:Register("sound", "alert-2", [[Interface\Addons\weizPVP\Sounds\alert-2.ogg]])
SM:Register("sound", "alert-3", [[Interface\Addons\weizPVP\Sounds\alert-3.ogg]])
SM:Register("sound", "alert-4", [[Interface\Addons\weizPVP\Sounds\alert-4.ogg]])
SM:Register("sound", "alert-5", [[Interface\Addons\weizPVP\Sounds\alert-5.ogg]])
SM:Register("sound", "alert-6", [[Interface\Addons\weizPVP\Sounds\alert-6.ogg]])
SM:Register("sound", "alert-7", [[Interface\Addons\weizPVP\Sounds\alert-7.ogg]])
SM:Register("sound", "alert-8", [[Interface\Addons\weizPVP\Sounds\alert-8.ogg]])
SM:Register("sound", "alert-9", [[Interface\Addons\weizPVP\Sounds\alert-9.ogg]])
SM:Register("sound", "alert-10", [[Interface\Addons\weizPVP\Sounds\alert-10.ogg]])
SM:Register("sound", "alert-11", [[Interface\Addons\weizPVP\Sounds\alert-11.ogg]])
SM:Register("sound", "alert-12", [[Interface\Addons\weizPVP\Sounds\alert-12.ogg]])
SM:Register("sound", "alert-13", [[Interface\Addons\weizPVP\Sounds\alert-13.ogg]])
SM:Register("sound", "alert-14", [[Interface\Addons\weizPVP\Sounds\alert-14.ogg]])
--> Sound Alarms
SM:Register("sound", "beep-1", [[Interface\Addons\weizPVP\Sounds\beep-1.ogg]])
SM:Register("sound", "beep-2", [[Interface\Addons\weizPVP\Sounds\beep-2.ogg]])
--> Sound Interface
SM:Register("sound", "interface-add", [[Interface\Addons\weizPVP\Sounds\interface-add.ogg]])
SM:Register("sound", "interface-remove", [[Interface\Addons\weizPVP\Sounds\binterface-remove.ogg]])
--> Sound Lockon
SM:Register("sound", "lockon-1", [[Interface\Addons\weizPVP\Sounds\lockon-1.ogg]])
--> Sound Music
SM:Register("sound", "music-lmfoa", [[Interface\Addons\weizPVP\Sounds\music-lmfoa.ogg]])
SM:Register("sound", "music-pistolero", [[Interface\Addons\weizPVP\Sounds\music-pistolero.ogg]])
--> Sound Notice
SM:Register("sound", "notice-1", [[Interface\Addons\weizPVP\Sounds\notice-1.ogg]])
SM:Register("sound", "notice-2", [[Interface\Addons\weizPVP\Sounds\notice-2.ogg]])
SM:Register("sound", "notice-3", [[Interface\Addons\weizPVP\Sounds\notice-3.ogg]])
--> Sound Positive
SM:Register("sound", "positive-1", [[Interface\Addons\weizPVP\Sounds\positive-1.ogg]])
--> Sound Voices
SM:Register("sound", "voice-aggro", [[Interface\Addons\weizPVP\Sounds\voice-aggro.ogg]])

-------------------------------------------------------------------------------
--=> Check DBs
-------------------------------------------------------------------------------
function weizPVP:CheckDatabase()
    --> Check for Player DB
    if weizPVP.db.profile.PlayerDB == nil then
        weizPVP.db.profile.PlayerDB = weizPVP.PlayerDB
    end

    --> Check for Settings
    if weizPVP.db.profile.Settings == nil then
        weizPVP.db.profile.Settings = weizPVP.Settings
    end
end

--=> Register Module Options
-------------------------------------------------------------------------------
-- function weizPVP:RegisterModuleOptions(name, optionTbl, displayName)
--     weizPVP.options.args[name] = (type(optionTbl) == "function") and optionTbl() or optionTbl
--     weizPVP.optionsFrames[name] =
--         LibStub("AceConfigDialog-3.0"):AddToBlizOptions("weizPVP", displayName, "weizPVP", name)
-- end

--=> Setup Options
-------------------------------------------------------------------------------
-- function weizPVP:SetupOptions()
--     LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable("weizPVP", weizPVP.options)
--     LibStub("AceConfig-3.0"):RegisterOptionsTable("weizPVP Commands", weizPVP.optionsSlash, "weizPVP")
--     local ACD3 = LibStub("AceConfigDialog-3.0")

--     weizPVP.optionsFrames = {}
--     weizPVP.optionsFrames.weizPVP = ACD3:AddToBlizOptions("weizPVP", nil, nil, "General")
--     weizPVP.optionsFrames.DisplayOptions =
--         ACD3:AddToBlizOptions("weizPVP", "DisplayOptions", "weizPVP Option", "DisplayOptions")
--     weizPVP.optionsFrames.AlertOptions =
--         ACD3:AddToBlizOptions("weizPVP", "AlertOptions", "weizPVP Option", "AlertOptions")
--     weizPVP.optionsFrames.ListOptions = ACD3:AddToBlizOptions("weizPVP", "ListOptions", "weizPVP Option", "ListOptions")
--     weizPVP.optionsFrames.DataOptions = ACD3:AddToBlizOptions("weizPVP", "DataOptions", "weizPVP Option", "DataOptions")

--     weizPVP:RegisterModuleOptions("Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(weizPVP.db), "Profiles")
--     weizPVP.options.args.Profiles.order = -2
-- end

--=> Show Config
-------------------------------------------------------------------------------
-- function weizPVP:ShowConfig()
--     -- Open the profiles tab first so the menu expands
--     InterfaceOptionsFrame_OpenToCategory(weizPVP.optionsFrames.Profiles)
--     InterfaceOptionsFrame_OpenToCategory(weizPVP.optionsFrames.weizPVP)
-- end

--=> OnInitialize
-------------------------------------------------------------------------------
function weizPVP:OnInitialize()
    --> LOAD ACE3 DB
    weizPVP.db = LibStub("AceDB-3.0"):New("weizPVP_DB")

    --> Check Database
    weizPVP:CheckDatabase()

    --> Initialize Static DBs
    weizPVP:CreateAbilityDB()
    weizPVP:CreateSpecDB()

    --> Apply stored and set default options
    -- weizPVP.SetupOptions()

    --> Set Global Player Info
    weizPVP.PlayerLevel = UnitLevel("player") --> player's level
    weizPVP.CurrentRealm = GetRealmName() --> player's realm name
    weizPVP.CurrentCharName = GetUnitName("player") --> player's char name
    weizPVP.FactionName = select(1, UnitFactionGroup("player")) --> Horde/Alliance


    --> Minimap Icon
    -- weizPVP.db =
    --     LibStub("AceDB-3.0"):New(
    --     "weizPVP_DB",
    --     {
    --         profile = {
    --             minimap = {
    --                 hide = false
    --             }
    --         }
    --     }
    -- )
    -- LDBIcon:Register("weizPVP", weizPVP_LDB, weizPVP.db.profile.minimap)
    -- weizPVP:RegisterChatCommand("weizPVP", "minimap")
end

--=> Minimap Icon
-------------------------------------------------------------------------------
-- function weizPVP:CommandMiniMapIcon()
--     weizPVP.db.profile.minimap.hide = not weizPVP.db.profile.minimap.hide
--     if weizPVP.db.profile.minimap.hide then
--         miniMapIcon:Hide()
--     else
--         miniMapIcon:Show()
--     end
-- end

--> OnLoad
-------------------------------------------------------------------------------
function weizPVP:OnLoad()
end

--=> OnEnable
-------------------------------------------------------------------------------
function weizPVP:OnEnable()
    --> Timers
    weizPVP:ScheduleRepeatingTimer("ManageExpirations", 1) --> Manages the list expirations on the player list
    weizPVP:ScheduleRepeatingTimer("SaveAddonData", 10) --> Saves data every 10 seconds
    weizPVP:ScheduleRepeatingTimer("HighUsageCheck", 5) --> Check for high usage to reduce strain on gathering info

    --=> Registered Events
    -------------------------------------------------------------------------------
    --> DATA COLLECTION
    weizPVP:RegisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
    weizPVP:RegisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
    weizPVP:RegisterEvent("UNIT_FACTION", "ZoneChangedEvent")
    weizPVP:RegisterEvent("PLAYER_TARGET_CHANGED", "PlayerTargetEvent")
    weizPVP:RegisterEvent("UNIT_TARGET", "UnitTargetEvent")
    weizPVP:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
    weizPVP:RegisterEvent("PLAYER_DEAD", "PlayerDeadEvent")
    -- weizPVP:RegisterEvent("PLAYER_ALIVE", "PlayerAliveEvent")
    weizPVP:RegisterEvent("CHAT_MSG_CHANNEL_NOTICE", "ChannelNoticeEvent")
    weizPVP:RegisterEvent("UPDATE_MOUSEOVER_UNIT", "PlayerMouseoverEvent")
    weizPVP:RegisterEvent("PLAYER_FOCUS_CHANGED", "PlayerFocusEvent")

    --> ZONE REALTED
    weizPVP:RegisterEvent("ZONE_CHANGED", "ZoneChangedEvent")
    weizPVP:RegisterEvent("ZONE_CHANGED_INDOORS", "ZoneChangedEvent")
    weizPVP:RegisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneChangedNewAreaEvent")
    weizPVP:RegisterEvent("PLAYER_ENTERING_WORLD", "PlayerEnteringWorldEvent")

    --> HEALTH UPDATES
    -- weizPVP:RegisterEvent("UNIT_HEALTH_FREQUENT", "UnitHealth") --> Faster updates, but more cpu consuming
    weizPVP:RegisterEvent("UNIT_HEALTH", "UnitHealth")
    weizPVP:RegisterEvent("PLAYER_REGEN_DISABLED", "EnteringCombat")
    weizPVP:RegisterEvent("PLAYER_REGEN_ENABLED", "LeavingCombat")
    weizPVP:RegisterEvent("PLAYER_LEVEL_UP", "PlayerLevelUp")
    weizPVP:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED", "SpellCastSucceeded")
    weizPVP:RegisterEvent("UNIT_AURA", "AuraUpdate")
    --> MAP
    weizPVP:RegisterEvent("MINIMAP_PING", "MinimapPing")
    --> Lockon
    -- weizPVP.LockonFrame:RegisterEvent("PLAYER_ENTERING_WORLD", "TargetChanged")
    -- weizPVP.LockonFrame:RegisterEvent("PLAYER_TARGET_CHANGED", "TargetChanged")
    -- weizPVP.LockonFrame:RegisterEvent("DISPLAY_SIZE_CHANGED", "DisplayChanged")
    -- weizPVP.LockonFrame:RegisterEvent("PLAYER_LOGIN", "DisplayChanged")
    -- weizPVP.LockonFrame:RegisterEvent("NAME_PLATE_UNIT_ADDED")
    -- weizPVP.LockonFrame:RegisterEvent("NAME_PLATE_UNIT_REMOVED")
    -- weizPVP.LockonFrame:RegisterEvent("DUEL_FINISHED")
    --!!-> DEBUG
    -- weizPVP:RegisterEvent("UPDATE_STEALTH", "UpdatedStealth")
end

--=> OnDisable
-------------------------------------------------------------------------------
function weizPVP:OnDisable()
    --=> uNRegistered Events
    -------------------------------------------------------------------------------
    --> ZONE RELATED
    weizPVP:UnregisterEvent("ZONE_CHANGED", "ZoneChangedEvent")
    weizPVP:UnregisterEvent("ZONE_CHANGED_INDOORS", "ZoneChangedEvent")
    weizPVP:UnregisterEvent("ZONE_CHANGED_NEW_AREA", "ZoneChangedNewAreaEvent")
    weizPVP:UnregisterEvent("PLAYER_ENTERING_WORLD", "PlayerEnteringWorldEvent")
    --> DATA COLLECTION
    weizPVP:UnregisterEvent("PLAYER_TARGET_CHANGED", "PlayerTargetEvent")
    weizPVP:UnregisterEvent("UNIT_TARGET", "UnitTargetEvent")
    weizPVP:UnregisterEvent("UPDATE_MOUSEOVER_UNIT", "PlayerMouseoverEvent")
    weizPVP:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED", "CombatLogEvent")
    weizPVP:UnregisterEvent("UNIT_FACTION", "ZoneChangedEvent")
    weizPVP:UnregisterEvent("PLAYER_DEAD", "PlayerDeadEvent")
    -- weizPVP:UnregisterEvent("PLAYER_ALIVE", "PlayerAliveEvent")
    weizPVP:UnregisterEvent("CHAT_MSG_CHANNEL_NOTICE", "ChannelNoticeEvent")
    weizPVP:UnregisterEvent("PLAYER_FOCUS_CHANGED", "PlayerFocusEvent")
    --> HEALTH UPDATES
    -- weizPVP:UnregisterEvent("UNIT_HEALTH_FREQUENT", "UnitHealth") --> Faster updates, but more cpu consuming
    weizPVP:UnregisterEvent("UNIT_HEALTH", "UnitHealth")
    weizPVP:UnregisterEvent("NAME_PLATE_UNIT_ADDED", "NameplateAdded")
    weizPVP:UnregisterEvent("NAME_PLATE_UNIT_REMOVED", "NameplateRemoved")
    weizPVP:UnregisterEvent("PLAYER_REGEN_DISABLED", "EnteringCombat")
    weizPVP:UnregisterEvent("PLAYER_REGEN_ENABLED", "LeftCombatEvent")
    weizPVP:UnregisterEvent("PLAYER_LEVEL_UP", "PlayerLevelUp")
    weizPVP:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED", "SpellCastSucceeded")
    weizPVP:UnregisterEvent("UNIT_AURA", "AuraUpdate")
    --> MAP
    weizPVP:UnregisterEvent("MINIMAP_PING", "MinimapPing")
    --> Lockon
    -- weizPVP.LockonFrame:UnregisterEvent("PLAYER_ENTERING_WORLD", "TargetChanged")
    -- weizPVP.LockonFrame:UnregisterEvent("PLAYER_TARGET_CHANGED", "TargetChanged")
    -- weizPVP.LockonFrame:UnregisterEvent("DISPLAY_SIZE_CHANGED", "DisplayChanged")
    -- weizPVP.LockonFrame:UnregisterEvent("PLAYER_LOGIN", "DisplayChanged")
    -- weizPVP.LockonFrame:UnregisterEvent("NAME_PLATE_UNIT_ADDED")
    -- weizPVP.LockonFrame:UnregisterEvent("NAME_PLATE_UNIT_REMOVED")
    -- weizPVP.LockonFrame:UnregisterEvent("DUEL_FINISHED")
    --!!-> DEBUG
    -- weizPVP:UnregisterEvent("UPDATE_STEALTH", "UpdatedStealth")
end

--=> Clear List Data
-------------------------------------------------------------------------------
function weizPVP:ClearListData()
    table_wipe(weizPVP.PlayerActiveCache)
    table_wipe(weizPVP.ActiveList)
    table_wipe(weizPVP.InactiveList)
    table_wipe(weizPVP.NearbyList)
    table_wipe(weizPVP.CurrentList)
    weizPVP:RefreshCurrentList()
    weizPVP:UpdateNearbyCount()
end

--=> Save Addon Data
-------------------------------------------------------------------------------
function weizPVP:SaveAddonData()
    weizPVP.db.profile.Settings = weizPVP.Settings
    weizPVP.db.profile.PlayerDB = weizPVP.PlayerDB
end

--=> MiniMap Toggle
-------------------------------------------------------------------------------
-- local mmIconVisable = true
-- function weizPVP:MiniMapToggle()
--     if mmIconVisable == true then
--         weizPVP.HeaderFrame:Hide()
--         weizPVP.CoreFrame:Hide()
--         mmIconVisable = false
--     else
--         weizPVP.HeaderFrame:Show()
--         weizPVP.CoreFrame:Show()
--         mmIconVisable = true
--     end
-- end

--=> Reset Profile
-------------------------------------------------------------------------------
-- function weizPVP:ResetProfile()
--     weizPVP.db.profile = weizPVP.Default_Profile.profile
--     weizPVP:CheckDatabase()
-- end

--=> Profile Change
-- -------------------------------------------------------------------------------
-- function weizPVP:HandleProfileChanges()
--     weizPVP:CreateMainWindow()
--     weizPVP:UpdateTimeoutSettings()
--     weizPVP:LockWindows(weizPVP.db.profile.Locked)
-- end

-- --=> Register Module Options
-- -------------------------------------------------------------------------------
-- function weizPVP:RegisterModuleOptions(name, optionTbl, displayName)
--     weizPVP.options.args[name] = (type(optionTbl) == "function") and optionTbl() or optionTbl
--     weizPVP.optionsFrames[name] =
--         LibStub("AceConfigDialog-3.0"):AddToBlizOptions("weizPVP", displayName, "weizPVP", name)
-- end

